/*
 * Copyright (c) 2009 by syntevo GmbH. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  o Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *  o Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 *  o Neither the name of syntevo GmbH nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.syntevo.openapi.gui.table;

import java.awt.*;
import java.awt.event.*;
import java.util.List;
import javax.swing.*;

/**
 * A sorted table. A table is defined by its table columns, see ITableColumn.
 * The table has a list of row objects. Every ITableColumn renderers a row
 * object from its perspective. Sorting is delagted to ITableColumn. If no
 * columns have been chosen for sorting, the 'natural' order of the row
 * object list will be used.
 *
 * By default, the table is in multi-selection mode; use setSingleSectionMode
 * to toggle selection behavior.
 */
public interface SortedTable<RO> {

	/**
	 * Returns the row objects displayed by the table
	 */
	List<RO> getRows();

	/**
	 * Set's the row objects displayed by the table
	 *
	 * @param rows  the row objects
	 */
	void setRows(List<RO> rows);

	/**
	 * Returns the row objects which have been selected (by the user or programmatically by setSelection).
	 */
	List<RO> getSelection();

	/**
	 * Set's the selected row object.
	 *
	 * @param selection  the row objects to select
	 */
	void setSelection(List<RO> selection);

	/**
	 * Set's the selection mode of the table (either multi-selection or
	 * single-selection).
	 *
	 * @param singleSelection  use true to have single-selection enabled
	 */
	void setSingleSelectionMode(boolean singleSelection);

	/**
	 * Adds a listener to the table which is invoked when the user
	 * double-clicks a row. Use the ActionEvent resp. getSelection()
	 * to get the double-clicked row.
	 *
	 * @param listener
	 */
	void addDoubleClickListener(ActionListener listener);

	/**
	 * Creates the actual Swing component which can then be added to -- for instance --
	 * a dialog. The returned component type is internal to SmartSVN, don't make any
	 * assumptions on this type. This method must be only called once.
	 *
	 * @param preferredSize  the preferred size of the overall table component
	 * @return the created component
	 */
	JComponent createComponent(Dimension preferredSize);
}
