/*
 * Copyright (c) 2002-2009 syntevo GmbH. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  o Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *  o Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 *  o Neither the name of syntevo GmbH nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.syntevo.q.gui.layout;

import java.awt.*;
import java.util.*;
import javax.swing.*;

/**
 * @author Thomas Singer
 */
public final class TableGridGroup {

	// Fields =================================================================

	private final Collection<TableGridColumnOrRow> members = new ArrayList<TableGridColumnOrRow>(4);
	private Collection<JComponent> additionalComponents;

	private transient Collection<Component> cachedComponents;

	// Setup ==================================================================

	public TableGridGroup() {
	}

	// Accessing ==============================================================

	public void addComponent(JComponent component) {
		if (component == null) {
			throw new IllegalArgumentException("expected not-null component");
		}

		if (additionalComponents == null) {
			additionalComponents = new LinkedList<JComponent>();
		}

		additionalComponents.add(component);
		invalidateCache();
	}

	// package local
	Collection<Component> getComponents() {
		if (cachedComponents == null) {
			cachedComponents = new ArrayList<Component>();
			for (final TableGridColumnOrRow columnOrRow : members) {
				cachedComponents.addAll(columnOrRow.getComponents());
			}
			if (additionalComponents != null) {
				cachedComponents.addAll(additionalComponents);
			}
		}
		return cachedComponents;
	}

	// package local
	void add(TableGridColumnOrRow columnOrRow) {
		if (columnOrRow == null) {
			throw new NullPointerException("columnOrRow must not be null");
		}

		members.add(columnOrRow);
	}

	// package local
	void invalidateCache() {
		cachedComponents = null;
	}
}