package com.syntevo.plugin.jira.commit;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.swing.filechooser.FileFilter;

import com.syntevo.openapi.gui.*;
import com.syntevo.openapi.gui.dialog.*;
import com.syntevo.q.gui.layout.*;

/**
 * @author syntevo GmbH
 */
final class JiraClientCertificateDialog extends AbstractOkCancelDialog {

	// Fields =================================================================

	private final File certificate;
	private final String passphrase;
	private final boolean storePassphrase;
	private final GuiSpacings spacings;

	private JTextField certificateTextField;
	private JPasswordField passphraseField;
	private JCheckBox storePassphraseCheckBox;

	// Setup ==================================================================

	public JiraClientCertificateDialog(File certificate, String passphrase, boolean storePassphrase, GuiSpacings spacings) {
		this.certificate = certificate;
		this.passphrase = passphrase;
		this.storePassphrase = storePassphrase;
		this.spacings = spacings;

		getOkAction().setName("Login");
	}

	// Implemented ============================================================

	public String getTitle() {
		return "JIRA - Client Certificate";
	}

	public JComponent createComponent(Container parent) {
		final JComponent panel = new JPanel();
		parent.add(panel);

		final JLabel certificateLabel = JiraUiUtils.createLabel("Certificate", 'C');

		certificateTextField = JiraUiUtils.createTextField(null, 30, certificateLabel);
		setCertificateFile(certificate);

		final JButton certificateFileChooserButton = new JButton("Select...");
		certificateFileChooserButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				showFileChooser();
			}
		});

		final JLabel passphraseLabel = JiraUiUtils.createLabel("Passphrase", 'P');

		passphraseField = JiraUiUtils.createPasswordField(passphrase != null, passphraseLabel);

		storePassphraseCheckBox = new JCheckBox("Store passphrase (will be stored in plain text!)");
		storePassphraseCheckBox.setMnemonic('S');
		storePassphraseCheckBox.setSelected(storePassphrase);

		final TableGridLayoutFactory layout = new TableGridLayoutFactory(panel);
		layout.addColumnNoStretch();
		layout.addColumnNoStretch(spacings.getXRelated());
		layout.addColumnStretchOnResize();
		layout.addColumnNoStretch(spacings.getXRelated());
		layout.addColumnNoStretch();
		layout.addRowNoStretch();
		layout.addRowNoStretch(spacings.getYUnrelated());
		layout.addRowNoStretch();
		layout.addRowNoStretch(spacings.getYUnrelated());
		layout.addRowNoStretch();
		layout.addComponent(certificateLabel, 0, 0, false, false, false, true);
		layout.addComponent(certificateTextField, 0, 2, false, true, false, true);
		layout.addComponent(certificateFileChooserButton, 0, 4);
		layout.addComponent(passphraseLabel, 2, 0);
		layout.addComponent(passphraseField, 2, 2, 1, 3);
		layout.addComponent(storePassphraseCheckBox, 4, 0, 1, 5, true, true, true, false);
		return panel;
	}

	public void validate() throws DialogValidationFailedException {
		if (getCertificate() == null) {
			throw new DialogValidationFailedException("Please select a certificate.", certificateTextField);
		}
	}

	@Override
	public void dialogShowing() {
		super.dialogShowing();
		final JComponent componentToFocus = getCertificate() == null ? certificateTextField : passphraseField;
		componentToFocus.requestFocus();
	}

	// Accessing ==============================================================

	public File getCertificate() {
		final String text = certificateTextField.getText();
		if (text.length() == 0) {
			return null;
		}

		return new File(text);
	}

	public String getPassphrase() {
		return JiraUiUtils.getPassword(passphraseField, passphrase);
	}

	public boolean isStorePassphrase() {
		return storePassphraseCheckBox.isSelected();
	}

	// Utils ==================================================================

	private void showFileChooser() {
		final JFileChooser fileChooser = new JFileChooser();
		fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		fileChooser.setDialogType(JFileChooser.OPEN_DIALOG);
		fileChooser.setMultiSelectionEnabled(false);
		fileChooser.setFileFilter(new FileFilter() {
			@Override
			public boolean accept(File file) {
				if (file.isDirectory()) {
					return true;
				}

				final String name = file.getName().toLowerCase();
				return name.length() > 4 && name.endsWith(".p12");
			}

			@Override
			public String getDescription() {
				return "Certificate File";
			}
		});
		if (fileChooser.showOpenDialog(certificateTextField) != JFileChooser.APPROVE_OPTION) {
			return;
		}

		setCertificateFile(fileChooser.getSelectedFile());
		certificateTextField.requestFocusInWindow();
	}

	private void setCertificateFile(File file) {
		JiraUiUtils.setText(certificateTextField, file != null ? file.getAbsolutePath() : null);
	}
}